/*************************************************************************/
/*  _____________                                                        */
/* < yeahconsole >                                                       */
/*  -------------                                                        */
/*         \   ^__^                                                      */
/*          \  (oo)\_______                                              */
/*             (__)\       )\/\                                          */
/*                 ||----w |                                             */
/*                 ||     ||                                             */
/*                                                                       */
/*  Copyright (C) knorke                                                 */
/*                                                                       */
/*  This program is free software; you can redistribute it and/or modify */
/*  it under the terms of the GNU General Public License as published by */
/*  the Free Software Foundation; either version 2 of the License, or    */
/*  (at your option) any later version.                                  */
/*                                                                       */
/*  This program is distributed in the hope that it will be useful,      */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/*  GNU General Public License for more details.                         */
/*                                                                       */
/*  You should have received a copy of the GNU General Public License    */
/*  along with this program; if not, write to the Free Software          */
/*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            */
/*************************************************************************/

#include <X11/Xlib.h>
#include <X11/keysym.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* key grabbing stuff taken from evilwm*/
#define grab_keysym(w, mask, keysym) \
	XGrabKey(dpy, XKeysymToKeycode(dpy, keysym), mask, w, True, \
			GrabModeAsync, GrabModeAsync); \
	XGrabKey(dpy, XKeysymToKeycode(dpy, keysym), LockMask|mask, w, True, \
			GrabModeAsync, GrabModeAsync); \
	if (numlockmask) { \
		XGrabKey(dpy, XKeysymToKeycode(dpy, keysym), numlockmask|mask, \
				w, True, GrabModeAsync, GrabModeAsync); \
		XGrabKey(dpy, XKeysymToKeycode(dpy, keysym), \
				numlockmask|LockMask|mask, w, True, \
				GrabModeAsync, GrabModeAsync); \
	}

Display *dpy;
Window root;
Window win;
int screen;
int opt_x, opt_width, opt_height, opt_delay;
KeySym opt_key;
unsigned int modmask = 0;

void roll(int i);
void get_defaults(void);

int main(int argc, char *argv[])
{
    XSetWindowAttributes attrib;
    int i, j;
    char cmd[255];
    XModifierKeymap *modmap;
    unsigned int numlockmask = 0;
    Window termwin;
    int hidden = 1;
    if (!(dpy = XOpenDisplay(NULL))) {
        fprintf(stderr, " can not open dpy %s", XDisplayName(NULL));
    }
    screen = DefaultScreen(dpy);
    root = RootWindow(dpy, screen);
    get_defaults();
    attrib.override_redirect = True;
    attrib.background_pixel = BlackPixel(dpy, screen);
    win = XCreateWindow(dpy, root,
                        opt_x, -opt_height, opt_width, opt_height,
                        0, CopyFromParent, InputOutput, CopyFromParent, CWOverrideRedirect | CWBackPixel, &attrib);
    XSelectInput(dpy, win, SubstructureNotifyMask | EnterWindowMask | KeyPressMask);
    sprintf(cmd, " %d ", (int) win);
    if (!fork()) {
        execlp("xterm", "xterm", "-name", "yeahconsole", "-into", cmd, NULL);
        exit(1);
    }
    /* key grabbing stuff taken from evilwm */
    modmap = XGetModifierMapping(dpy);
    for (i = 0; i < 8; i++) {
        for (j = 0; j < modmap->max_keypermod; j++) {
            if (modmap->modifiermap[i * modmap->max_keypermod + j] == XKeysymToKeycode(dpy, XK_Num_Lock)) {
                numlockmask = (1 << i);
            }
        }
    }
    XFreeModifiermap(modmap);
    grab_keysym(root, modmask, opt_key);
    XMapWindow(dpy, win);
    XEvent event;
    while (1) {
        XMaskEvent(dpy, SubstructureNotifyMask, &event);
        if (event.type == ReparentNotify && event.xreparent.parent == win) {
            termwin = event.xreparent.window;
            XSetWindowBorderWidth(dpy, termwin, 0);
            XResizeWindow(dpy, termwin, opt_width, opt_height - 2);
            break;
        }
    }
    while (1) {
        XNextEvent(dpy, &event);
        switch (event.type) {
        case EnterNotify:
            XSetInputFocus(dpy, win, RevertToPointerRoot, CurrentTime);
            break;
        case KeyPress:
            if (XKeycodeToKeysym(dpy, event.xkey.keycode, 0) == opt_key) {
                if (!hidden) {
                    roll(0);
                    XSetInputFocus(dpy, root, RevertToPointerRoot, CurrentTime);
                    hidden = 1;
                } else {
                    XRaiseWindow(dpy, win);
                    roll(-opt_height);
                    XSetInputFocus(dpy, termwin, RevertToPointerRoot, CurrentTime);
                    hidden = 0;
                }
            }
            break;
        case UnmapNotify:
            if (event.xunmap.window == termwin)
                exit(0);
            break;
        }
    }
    return 0;
}

void roll(int i)
{
    int x = 5;
    if (i == 0)
        x = -5;
    while (1) {
        i += x;
        XMoveWindow(dpy, win, opt_x, i);
        XSync(dpy, False);
        usleep(opt_delay * 100);
        if (i == 0 || i == -opt_height)
            break;
    }
}

void get_defaults()
{
    char *opt;
    opt = XGetDefault(dpy, "yeahconsole", "screenWidth");
    opt_width = opt ? atoi(opt) : DisplayWidth(dpy, screen);
    opt = XGetDefault(dpy, "yeahconsole", "height");
    opt_height = opt ? atoi(opt) : 200;
    opt = XGetDefault(dpy, "yeahconsole", "xOffset");
    opt_x = opt ? atoi(opt) : 0;
    opt = XGetDefault(dpy, "yeahconsole", "aniDelay");
    opt_delay = opt ? atoi(opt) : 40;
    opt = XGetDefault(dpy, "yeahconsole", "toggleKey");
    opt_key = opt ? XStringToKeysym(opt) : XK_y;
    opt = XGetDefault(dpy, "yeahconsole", "modifier");
    if (opt) {
        if (strstr(opt, "Control"))
            modmask = modmask | ControlMask;
        if (strstr(opt, "Alt"))
            modmask = modmask | Mod1Mask;
        if (strstr(opt, "Win"))
            modmask = modmask | Mod4Mask;
        if (strstr(opt, "None")) {
            modmask = 0;
            return;
        }
    }
    if (!modmask)
        modmask = ControlMask | Mod1Mask;
}
